using System;
using System.Collections.Generic;
using DomainObjects.Facade.Domain;
using DomainObjects.Proxy;
using DomainObjects.Facade;
using DomainObjects.Facade.Command;
using DomainObjects.Facade.Transaction;

namespace com.diagast.testdbmapping {
    [Transaction(TransactionOption.Required)]
    public sealed class DomainObjectsDbAccess : Service<DomainObjectsDbAccess> {

        public EditableObject CreateObjectInDB<EditableObjectType>(EditableObject obj) where EditableObjectType : EditableObject {
            PersistenceFacade.Persist(obj); //call to persist because the object is created outside the Service method scope.
            return obj.Manage<EditableObjectType>();  
        }

        public void UpdateObjectInDB(EditableObject obj) {
            BindUtil.BindToTransaction(obj);
        }

        public void DeleteObjectInDB(EditableObject obj) {
            obj.Delete();
        }

        public EditableObject FindObjectInDBWithPrimaryKey<EditableObjectType>(object primaryKey) where EditableObjectType : EditableObject {
            return (EditableObject)QueryFacade.FindObjectByPrimaryKey<EditableObjectType>(primaryKey);
        }

        public List<EditableObjectType> FindAllDBObjects<EditableObjectType>() where EditableObjectType : EditableObject {
            Criteria criteria = new Criteria();
 //           criteria.AddOrderBy(Types.Demande.Field._sampleid);
            return QueryFacade.FindObjectSet<EditableObjectType>(criteria);
        }
    }
}
