/**
 *
 */
package org.openmobileis.bundle.osgi.terminal;

import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Properties;

import org.openmobileis.bundle.osgi.services.BunbleContextManager;
import org.openmobileis.bundle.osgi.services.TemplateRetrieverServiceImpl;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.embedded.webserver.templates.TemplateRetrieverManagerService;
import org.openmobileis.embedded.webserver.templates.TemplateRetrieverService;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.core.ProfileModuleRubric;
import org.openmobileis.module.core.XmlModuleParser;
import org.openmobileis.modules.profiles.terminal.RubricLoader;
import org.openmobileis.services.Service;
import org.openmobileis.services.common.ServiceManager;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

/**
 * @author phil
 *
 */
public final class ModuleBundleActivator implements BundleActivator {
	ApplicationContextManager manager = ApplicationContextManager.getManager();
	private Array registeredServiceList;

	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bundleContext) throws Exception {
	  //init class loading
	  ClassLoader bundleLoader = this.getClass().getClassLoader();
	  ClassLoader oldLoader = Thread.currentThread().getContextClassLoader();
	  Thread.currentThread().setContextClassLoader(bundleLoader);
	  BunbleContextManager.getManager().setBundleContext(bundleContext);

		//init log
		Properties props = new Properties();
		props.put("org.openmobileis.common.log.file", System.getProperty("user.dir") + "/WEB-INF/log/openmislog.txt");
		File file = new File(System.getProperty("user.dir") + "/WEB-INF/log");
		if (file.exists())
			FileOpenCloseLogManager.registerLogManager(props); //log to a file
		else
			LogManager.registerLogManager(null); //log to a file

		//joint webserver session if init made in another thread than the web server.
		SessionContextManager.getManager().joinSessionContext("OPENMOBILEISSESSION");
		
		//init template management
		try {
			ServiceReference[] refs = bundleContext.getServiceReferences(TemplateRetrieverManagerService.class.getName(), null);
			if (refs != null) {
				TemplateRetrieverManagerService templatemanager = (TemplateRetrieverManagerService) bundleContext.getService(refs[0]);
		  	String rootTemplatePath = PropertiesManager.getManager().getProperty("org.openmobileis.services.templatesDir");
		  	TemplateRetrieverService retriever = new TemplateRetrieverServiceImpl(bundleContext.getBundle().getSymbolicName(), rootTemplatePath);
				templatemanager.registerTemplateRetriever(retriever);
				bundleContext.ungetService(refs[0]);
				refs[0] = null;
			}
		} catch (Throwable ex) {
			LogManager.traceError(0, ex);
		}

		//init module
		// get module XML
		InputStream in = bundleLoader.getResourceAsStream("/"+bundleContext.getBundle().getSymbolicName()+".xml");
		System.out.println("XML "+"/"+bundleContext.getBundle().getSymbolicName()+".xml in :"+in);
		XmlModuleParser parser = new XmlModuleParser(new InputStreamReader(in));
		parser.parse();
		ProfileModule module = parser.getModuletoload();
		this.notifyModuleLoading(module);
	  Thread.currentThread().setContextClassLoader(oldLoader);

	}

	/* (non-Javadoc)
	 * @see org.openmobileis.module.terminal.ModuleManagerListener#notifyModuleLoading(org.openmobileis.module.core.ProfileModule)
	 */
	public void notifyModuleLoading(ProfileModule module) throws ServiceException {
		Array rubList = module.getModuleRubrics();
		for (int i=0; i<rubList.size(); i++)	{
			ProfileModuleRubric rubric = (ProfileModuleRubric) rubList.get(i);
			String loaderClassName = rubric.getLoaderClass();
			if (loaderClassName != null)	{
				try	{
					RubricLoader loader = (RubricLoader) Thread.currentThread().getContextClassLoader().loadClass(loaderClassName).newInstance();
					this.loadRubricLoader(loader);
				} catch (Throwable ex)	{
					LogManager.traceError(0, "StaticModuleManagerListener notifyModuleLoading : Exception during module "+module.getName()+" loading :"+ex.getMessage());
					throw new ServiceException(ex);
				}
			}
		}


	}

  private void loadRubricLoader(RubricLoader rubricLoader) throws ServiceException  {
    rubricLoader.preLoadingInit();
    // load services
    Array serviceList = rubricLoader.loadService();
    registeredServiceList = new Array();
    for (int i=0; i<serviceList.size(); i++)  {
      Service service = (Service)serviceList.get(i);
      registeredServiceList.add(service.getServiceUri());
      ServiceManager.getManager().loadService(service);
    }
    // load listener
    rubricLoader.postLoadingInit();
  }
  
  private void unregisterServices()	{
    for (int i=0; i<registeredServiceList.size(); i++)  {
      String serviceURI = (String)registeredServiceList.get(i);
      ServiceManager.getManager().unloadService(serviceURI);
    }
  }

	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bundleContext) throws Exception {
		this.unregisterServices();
	}

}
