/**
 *
 */
package org.openmobileis.common.util;

import java.io.IOException;

/**
 * @author phil
 *
 */
public interface PropertiesService {

	/**
	 * return the property with specified key from loaded properties.
	 * return default value if not found.
	 *
	 * @param key : the property key to get
	 * @param defaultValue : default value if key is not found
	 * @return the property with specified key or default value if not found.
	 */
	public String getProperty(String key, String defaultValue);

	/**
	 * return the property with specified key from loaded properties.
	 * return null if not found.
	 *
	 * @param key : the property key to get
	 * @return the property with specified key or null if not found.
	 */
	public String getProperty(String key);

	/**
	 * add the specified key,value to the manager properties
	 *
	 * @param key : the property key
	 * @param value : value of the property
	 */
	public void addProperty(String key, String value);


	/**
	 * remove all resource manager properties
	 */
	public void removeAllProperties();


	/**
	 * load a property file inside the classpath.
	 * if the path is /org/openmobileis/file.properties
	 * First look in the classpath for  /properties/org/openmobileis/file.properties
	 * then in /org/openmobileis/file.properties.
	 * If not found throw IOException
	 *
	 * @param filename : properties file name.
	 * @exception IOException : throws if the file is not found
	 */
	public void addPropertiesFile(String filename) throws IOException;

	/**
	 * load a property file from the file system
	 * If not found throw IOException
	 *
	 * @param filename : properties file name.
	 * @exception IOException : throws if the file is not found
	 */
	public void addPropertiesFileFromFilePath(String filename) throws java.io.IOException;

	/**
	 * read the property file inside the classpath.
	 * Does not load it
	 * if the path is /org/openmobileis/file.properties
	 * First looks in the classpath for  /properties/org/openmobileis/file.properties
	 * then for /org/openmobileis/file.properties.
	 * If not found throw IOException
	 *
	 * @param filename : properties file name.
	 * @exception IOException : throws if the file is not found
	 */
	public java.util.Properties getProperties(String prop_file) throws java.io.IOException;

	/**
	 * read the property file from the file system
	 * Does not load it
	 *
	 * @param filename : properties file name.
	 * @exception IOException : throws if the file is not found
	 */
	public java.util.Properties getPropertiesFromPath(String prop_file)throws java.io.IOException;

	/**
	 * return all the loaded properties
	 *
	 * @return the loaded properties
	 */
	public java.util.Properties getProperties();




}
