/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */

package org.openmobileis.embedded.util;

import org.openmobileis.common.context.ApplicationContext;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.util.exception.OpenMISException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Helper class to access native OS specific task that is not correcly
 * done by the JVM.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class SystemAPI {
	private static SystemAPI manager;

	private static ISystemAPI api;

	private SystemAPI()	{

		if (api == null){
			try {
				ApplicationContext context =  ApplicationContextManager.getManager().getApplicationContext();
				if (context.getPlateform().getOS().equals(Plateform.POCKETPCOS))	{
					api = new PPCSystemAPI();
				} else	{
					api = new DefaultSystemAPI();
				}
			} catch (Throwable exi) {
				exi.printStackTrace();
			}
		}
	}

	public static void registerSystemAPI(ISystemAPI api)	{
		SystemAPI.api = api;
	}

	public static SystemAPI getManager()	{
		if (manager == null) {
			manager = new SystemAPI();
			ApplicationContextManager.getManager().addManager(manager);
		}
		return manager;
	}

  /**
  * Execute a program on mobile and wait until the program ended
  * @param: String path of the program
  * @param: Arguments separated by space
  */
  public int execProgramAndWait(String path, String args) throws OpenMISException {
	  return api.execProgramAndWait(path, args);
  }

  /**
  * Execute a program on mobile.
  * @param: String path of the program
  * @param: Arguments separated by space
  */
  public int execProgram (String path, String args) throws OpenMISException	{
	  return api.execProgram(path, args);
  }

  /**
  * Get time stamp, same behavior as system.currentTimeMillis in milli
  * Used to fix Creme V3.1 Bug
  */
  public long currentTimeInMillis() {
	  return System.currentTimeMillis();
  }
  
  public String getSystemUniqueID(){
	  return api.getSystemUniqueID();
  }

  public static void main (String[] args) {
    try {
        int i = SystemAPI.getManager().execProgramAndWait("\\Windows\\calc.exe","");
        System.out.println(i);
        i = SystemAPI.getManager().execProgram("\\Windows\\calendar.exe","");
        System.out.println(i);
      } catch (Exception e) {
        e.printStackTrace();
      }
  }
}