/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.embedded.webserver.templates;

import java.io.File;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;

import freemarker.template.FileTemplateCache;
import freemarker.template.Template;

/**
 * @author phil
 *
 */
public final class FileTemplateRetrieverService implements TemplateRetrieverService {
  private String rootTemplatePath;
  //use to retreive template from file system
  private FileTemplateCache fileTemplateCache = null;

	/**
	 * 
	 */
	public FileTemplateRetrieverService() {
  	rootTemplatePath = PropertiesManager.getManager().getProperty("org.openmobileis.services.templatesDir");
  	fileTemplateCache = new FileTemplateCache(rootTemplatePath);
  	fileTemplateCache.setFilenameSuffix("htm");
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.embedded.webserver.templates.TemplateRetrieverService#getModuleName()
	 */
	public String getModuleName() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.embedded.webserver.templates.TemplateRetrieverService#getServiceTemplate(java.lang.String)
	 */
	public Template getServiceTemplate(String relatifPath) {
    //use template cache to allow template include.
    IntlResourceManager resourceManager = IntlResourceManager.getManager();
    String newrelatifPath = resourceManager.getLocalizedFileName(relatifPath, false);
	    //test if the file exist. If not use not localized version
	    File file = new File(rootTemplatePath+newrelatifPath);
	    if (!file.exists()) newrelatifPath = relatifPath;
    
   // LogManager.traceDebug(0, "TemaplateManager gettemplate newrelatifPath "+newrelatifPath);
   // LogManager.traceDebug(0, "TemaplateManager gettemplate relatifPath "+relatifPath);
   // LogManager.traceDebug(0, "TemaplateManager gettemplate rootTemplatePath "+rootTemplatePath);
    
    return (Template)fileTemplateCache.getItem(newrelatifPath);
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.embedded.webserver.templates.TemplateRetrieverService#isTemplateExist(java.lang.String)
	 */
	public boolean isTemplateExist(String relatifPath) {
    IntlResourceManager resourceManager = IntlResourceManager.getManager();
    String newrelatifPath = resourceManager.getLocalizedFileName(relatifPath, false);
	  //test if the file exist. If not use not localized version
	  File file = new File(rootTemplatePath+File.separator+newrelatifPath);
	  if (!file.exists()) newrelatifPath = relatifPath;
	  file = new File(rootTemplatePath+File.separator+newrelatifPath);
	  return file.exists();
	}

}
