/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 *
 */

package org.openmobileis.embedded.webserver.templates;

import javax.servlet.http.HttpServletResponse;

import freemarker.template.TemplateModelRoot;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base interface for template management.
 * Template management are delegated to delegate. Delegate must be registered to the TemplateManager before its activation
 * (call to getManager() method).
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public interface TemplateManagerDelegate {

  /**
  * init template and cache management.
  * cache template is use to detect cache update during synchronization
  */
  public void init();

  public void updateTemplateWithNavigationBarData(Object modelRoot, String data);

  /**
   * initialize template structure for this request.
   */
  public TemplateModelRoot getTemplateModelRoot() ;

/*
  * Get the template called templateName (load it if it is not already done),
  * replace all variables tree of TemplateModelRoot in the template.
  * Write the HTTP response in the output stream with a basic header (mime type + status)
  * @param: String templateName the name of the template to use
  * @param: Hashtable variables (varName -> variable value) pairs
  * @param: HttpServletResponse response
  * @return: none
  * @OpenMSPException : IOEXception if an error occurs during sending data
  * @OpenMSPException : TemplateNotFoundException if the template is not found
  */
  public void sendResponse (String templateName, TemplateModelRoot modelRoot, HttpServletResponse res) throws java.io.IOException, TemplateNotFoundException;

}



