/**
 *
 */
package org.openmobileis.services;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.common.ServiceManager;

import freemarker.template.TemplateModelRoot;

/**
 * Description : use to define a sequence of StepService with  next and prev.
 * Service are added and call with the user form MultiStepMainServiceURI/stepservicename.
 *
 */
public abstract class MultiStepMainService extends TemplateService implements EditService {
	protected HashMap serviceMap;
	/**
	 *
	 */
	public MultiStepMainService() {
		serviceMap = new HashMap(10);
	}

	public String runTemplate (HttpServletRequest req, HttpServletResponse res , TemplateModelRoot templateData) throws ServletException, IOException {
		try	{
			String lasttag = req.getParameter("last");
			if (lasttag != null && lasttag.length() == 0) lasttag = null;
			String nexttag = req.getParameter("next");
			if (nexttag != null && nexttag.length() == 0) nexttag = null;
			String forcetag = req.getParameter("stepname");
			if (forcetag != null && forcetag.length() == 0) forcetag = null;
			String stepName = this.getReqStepName(req);
			String redirect = req.getParameter("redirectURL");
			HttpSession session = req.getSession(true);

			//remove session data if first call
			boolean firstcall = this.isFirstCall(req);
			if (firstcall)	{
				this.removeSessionData(req);
			}
			Object sessionDatas = this.getSessionData(req);
			if (sessionDatas == null)	{ //first call for create.
				sessionDatas = this.createSessionObject(req);
				if (sessionDatas == null)	{
					StepEditService edit = (StepEditService) serviceMap.get(this.getReqStepName(req));
					return edit.getTemplateName(sessionDatas, req);
				}
				this.setSessionData(req, sessionDatas);
			}

			//manage redirect
			if(redirect != null && !firstcall){
				//put the params request in the session and do redirect service
			//	HttpSession session = req.getSession(true);
			//	session.setAttribute(this.getSessionDataTempName(), req.getParameterMap());

				Object red = session.getAttribute(this.getSessionDatasName()+"redirect");
				if (red == null)	{ // one redirect.
					// store step data before redirect.
					StepEditService edit = (StepEditService) serviceMap.get(stepName);
					if (edit != null)	{
						edit.fillSessionObjectWithRequestData(sessionDatas, req);
					}
					session.setAttribute(this.getSessionDatasName()+"redirect", "true");
					ServiceManager.getManager().redirectToServiceURI(redirect, req, res);
					return null;
				}
			}
			session.removeAttribute(this.getSessionDatasName()+"redirect");

/*			StepEditService currentEdit = this.getCurrentStep(req);
			if (currentEdit != null){
				currentEdit.fillSessionObjectWithRequestData(sessionDatas, req);
				if (!currentEdit.validateSessionStepData(sessionDatas, req, templateData))	{ //go to next step
					currentEdit.fillTemplateWithSessionData(sessionDatas, templateData);
					return currentEdit.getTemplateName();
				}
			} */

			StepEditService edit = (StepEditService) serviceMap.get(stepName);
			if (edit != null)	{
				edit.fillSessionObjectWithRequestData(sessionDatas, req);
				String nextStep = null;
				if (lasttag != null) nextStep = edit.getLastStepName(sessionDatas, req);
				else if (nexttag != null) nextStep = edit.getNextStepName(sessionDatas, req);
				else if (forcetag != null) nextStep = forcetag;
				else nextStep = edit.getStepName();
				if (nextStep != null)	{
					if (!nextStep.equals(edit.getStepName()))	{ //validate before change step.
						if (!edit.validateSessionStepData(sessionDatas, req, templateData))	{ //go to next step
							edit.fillTemplateWithSessionData(sessionDatas, req, templateData);
							return edit.getTemplateName(sessionDatas, req);
						}
						edit = (StepEditService) serviceMap.get(nextStep);
					}
					edit.fillTemplateWithSessionData(sessionDatas, req, templateData);
					return edit.getTemplateName(sessionDatas, req);
				} else	{ //validate if next step == null
					if (!edit.validateSessionStepData(sessionDatas, req, templateData))	{ //validate current edit
						edit.fillTemplateWithSessionData(sessionDatas, req, templateData);
						return edit.getTemplateName(sessionDatas, req);
					}
					this.storeSessionObjectInDB(sessionDatas, req, templateData);
					this.removeSessionData(req);
					//remove next and redirect from request
					Map params = req.getParameterMap();
					params.remove("last");
					params.remove("next");
					params.remove("redirectURL");
					ServiceManager.getManager().redirectToServiceURI(this.getDisplayServiceURI(), req, res);
					return null;
				}
			} else	{ //end no step => validate
				this.storeSessionObjectInDB(sessionDatas, req, templateData);
				this.removeSessionData(req);
				ServiceManager.getManager().redirectToServiceURI(this.getDisplayServiceURI(), req, res);
				return null;
			}
		} catch (Exception ex) {
			LogManager.traceError(0, ex);
			this.removeSessionData(req);
			throw new ServletException(ex);
		}
	}

	public void addSequenceService(StepEditService service)	{
		serviceMap.put(service.getStepName(), service);
	}

	public boolean hasSessionData(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		return session.getAttribute(this.getSessionDatasName()) != null;

	}

	protected Object getSessionData(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		return session.getAttribute(this.getSessionDatasName());
	}

	protected void setSessionData(HttpServletRequest req, Object sessionDatas)	{
		HttpSession session = req.getSession(true);
		session.setAttribute(this.getSessionDatasName(), sessionDatas);
	}


	public void removeSessionData(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		session.removeAttribute(this.getSessionDatasName());
		session.removeAttribute(this.getSessionDatasName()+"stepname");
		session.removeAttribute(this.getSessionDatasName()+"redirect");
	}

/*	protected StepEditService getCurrentStep(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		return (StepEditService) session.getAttribute(this.getSessionDatasName()+"stepname");
	}

	protected void setCurrentStep(HttpServletRequest req, StepEditService step)	{
		HttpSession session = req.getSession(true);
		session.setAttribute(this.getSessionDatasName()+"stepname", step);
	} */

	private String getReqStepName(HttpServletRequest req)	{
		String serviceURI = req.getRequestURI();
		int index = this.getServiceUri().length();
		String stepName = "";
		if ((index+1)< serviceURI.length() )	{
			stepName = serviceURI.substring(index+1, serviceURI.length());
		}
		index = stepName.indexOf("?");
		if (index != -1)	{
			stepName = stepName.substring(0, index);
		}
		return stepName;
	}

	/**
	 * Store the session data object.
	 * @param sessionDatas
	 * @param req
	 * @param templateData
	 * @throws ServiceException
	 */
	protected abstract void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException;

	protected abstract String getSessionDatasName();
	protected abstract Object createSessionObject(HttpServletRequest req) throws ServiceException;
	protected abstract String getDisplayServiceURI();
	protected abstract boolean isFirstCall(HttpServletRequest req);

}
