/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2007 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */
package org.openmobileis.synchro.openmsp.client.conduit;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;

import HTTPClient.AuthorizationInfo;
import HTTPClient.CookieModule;
import HTTPClient.HTTPConnection;
import HTTPClient.NVPair;

/**
 * @author phil
 *
 */
public final class HTTPConnectionFactory {
	public static HTTPConnection createConnection(String url, int port, String userAgent)	{
		HTTPConnection con = null;
		String socks = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.address");
		//			LogManager.traceDebug(0, "proxy :"+proxy);
		if (socks != null) {
			String sport = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.port");
			if (sport != null) {
				int socksport = Integer.parseInt(sport);
				HTTPConnection.setSocksServer(socks, socksport);
				String login = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.login");
				String pass = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.pass");
				String realm = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.realm");
				if (login != null && pass != null && realm!=null) {
					AuthorizationInfo.addBasicAuthorization(socks, socksport, realm, login, pass);
				}
			}
		}
		String proxy = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.address");
		//			LogManager.traceDebug(0, "proxy :"+proxy);
		if (proxy != null) {
			String pport = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.port");
			if (pport != null) {
				int proxyport = Integer.parseInt(pport);
				HTTPConnection.setProxyServer(proxy, proxyport);
				String login = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.login");
				String pass = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.pass");
				String realm = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.realm");
				if (login != null && pass != null && realm!=null) {
					AuthorizationInfo.addBasicAuthorization(proxy, proxyport, realm, login, pass);
				}
			}
		}
		con = new HTTPConnection(url, port);
		//manage proxy server

		CookieModule.setCookiePolicyHandler(null);
		NVPair headerData[] = new NVPair[1];
		headerData[0] = new NVPair("User-Agent", userAgent);
		con.setDefaultHeaders(headerData);
		return con;
	}

	public static String getDefaultUserAgent ()	{
    String usera = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.useragent");
    if (usera == null)  {
      Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
      StringBuffer useragent = new StringBuffer("OpenMSPPlug-");
      useragent.append(plateform.getOpenMobileISMajorVersion()).append('.').append(plateform.getOpenMobileISMinorVersion()).append('-');
      useragent.append(plateform.getOS()).append('[').append(plateform.getOSVersion()).append("]-");
      String language = IntlResourceManager.getManager().getManagerLocaleString();
      useragent.append(language);
      usera = useragent.toString();
    }
    return usera;
	}

}
