/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.util;

import java.io.*;

import org.openmobileis.common.util.UniqueIdGenerator;
import org.openmobileis.common.util.file.FileUtilities;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: memory file
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class FileStoredOpenMISFile implements OpenMISFile {
  static final long serialVersionUID = 5521257935120563452L;
  public static final String tempDir = "synchrotemp";
  protected String fileName;
  protected boolean copy = true;
  protected java.io.InputStream inputstream = null;

  /**
   * file path. End with path separator
   */
  protected String filePath;
  protected String tempFileName;
  protected long filedate;
  
  protected FileStoredOpenMISFile(){}
  
  /** 
   * specified data stream is close during reading.
   * @param data
   * @throws IOException
   */
  protected FileStoredOpenMISFile(InputStream data) throws IOException {
	filedate = System.currentTimeMillis();
	this.copy = true;
	tempFileName = System.getProperty("user.dir")+File.separator+FileStoredOpenMISFile.tempDir+File.separator+UniqueIdGenerator.getManager().getNewStringID();
    InputStream sourceFile = new BufferedInputStream(data);
    try {
      OutputStream destFile = new  BufferedOutputStream(new FileOutputStream(tempFileName));
      try {
        byte[] buffer = new byte[1024];
        int bytes_read;
        while(( bytes_read = sourceFile.read(buffer)) != -1) 
          destFile.write(buffer, 0, bytes_read);
      } finally {
        destFile.flush();
        destFile.close();
      }
    } finally {
      sourceFile.close();
    }
  }

  /**
   * specified data stream is close during reading.
   * @param completeName
   * @param data
   * @throws IOException
   */
  public FileStoredOpenMISFile(String completeName, InputStream data) throws IOException {
    this(data);
    completeName = FileUtilities.convertFileNameToSystem(completeName);
    fileName = FileUtilities.getNameFromCompleteFileName(completeName);
    filePath = FileUtilities.getPathFromCompleteFileName(completeName);
 }

  public FileStoredOpenMISFile(String filepath, String filename) throws IOException {
	    this.fileName = filename;
	    this.filePath = FileUtilities.convertFileNameToSystem(filepath);  
	    this.tempFileName = FileUtilities.convertFileNameToSystem(filepath+filename);
	    this.copy=false;
  }

  public FileStoredOpenMISFile(String name, String path, InputStream data) throws IOException {
    this(data);
    fileName = name;
    filePath = path;
  }

  /**
   * return the name of the file. The name doesnot contain file path.
   */
  public String getFileName() {
    return fileName;
  }

  /**
   * return file data as a stream
   */
  public java.io.InputStream getFileDataStream() throws IOException {
	  if (this.inputstream != null) this.inputstream.close();
	  this.inputstream = new FileInputStream(tempFileName);
	  return new BufferedInputStream(this.inputstream);
  }

  /**
   * return the path of the file without the file name. The path end with system separator.
   */
  public String getFilePath() {
    return filePath;
  }

  public long getFileDate() {
    return filedate;
  }

  public void setFileDate(java.util.Date date)  {
    filedate = date.getTime();
  }

  /**
   * return the complete file name of the file (ie: path+separator+name)
   */
  public String getFileCompleteName() {
    return filePath+fileName;
  }


  /**
   * return the file data length
   */
  public long getFileLength()  {
	  if (tempFileName != null)	{
		  File file = new File(tempFileName);
		  return file.length();
	  }
	  return 0;
  }

  /**
   * return file data as a byte array
   */
  public byte[] getFileData() throws IOException {
	  if (tempFileName != null)	{
		  return FileUtilities.readFile(tempFileName);
	  }
	  return new byte[0];
  }
  
  public void removeTempFile()throws IOException	{
	  if (this.inputstream != null) this.inputstream.close();
	  this.inputstream = null;
	  if (tempFileName != null && copy)	{
		  File file = new File(tempFileName);
		  file.delete();
	  }
  }

  public int hashCode() {
    return getFileCompleteName().hashCode();
  }

  public boolean equals(Object obj) {
    if (obj instanceof FileStoredOpenMISFile)  {
      return ((FileStoredOpenMISFile)obj).getFileCompleteName().equals(this.getFileCompleteName());
    }
    return false;
  }



}